clc
clear
close all

addpath './tsp/'
addpath './functional'

import opitimization.single_opitimization;
import transform.longlat2meter;

pathdir = '.\2020.04.25';

no_array = 1:9;

height_pathes = arrayfun(@(i) [pathdir,'\0',num2str(i),'0',num2str(i),'\0',num2str(i),'0',num2str(i),'̵߳.csv'],...
    no_array,...
    "UniformOutput",...
    false)';

edge_pathes = arrayfun(@(i) [pathdir,'\0',num2str(i),'0',num2str(i),'\0',num2str(i),'0',num2str(i),'߽.csv'],...
    no_array,...
    "UniformOutput",...
    false)';

earth_redius = 6371000;
origin = [127.1563; 42.3230];

height_maps = arrayfun(@(i) get_height_map(i{1}, earth_redius, origin), height_pathes, "UniformOutput", false);
edge_maps = arrayfun(@(i) get_edge_map(i{1}, earth_redius, origin), edge_pathes, "UniformOutput", false);
best_solutions = arrayfun(@(i) get_single(height_maps{i}, edge_maps{i},46, i), no_array, "UniformOutput", false);

disp("best solution")
arrayfun(@(i) disp("angle " + i{1}(1) + " length " + i{1}(2) + " entrance " + mat2str(i{1}(3:4))+ " export " + mat2str(i{1}(5:6))),...
    best_solutions');
arrayfun(@(i) main_plot(height_maps{i}(:, 1),...
    height_maps{i}(:, 2),...
    height_maps{i}(:, 3),...
    edge_maps{i}(:, 1),...
    edge_maps{i}(:, 2),...
    100,...
    best_solutions{i}(1)),...
    no_array,"UniformOutput", false);

farms = arrayfun(@(i) Farm(i{1}(3:4), i{1}(5:6)), best_solutions');
distance_matrix = farm_distance_matrix(farms);
best_path = tsp(distance_matrix);
disp("best path is" + mat2str(best_path))
figure(100);
hold on

for i = no_array(1:end - 1)
    this_farm = farms(best_path(i));
    next_farm = farms(best_path(i + 1));
    
    this_edge_map = edge_maps{best_path(i)};
    lines = [this_edge_map(1: end - 1, :),  this_edge_map(2: end, :)];
    next_edge_map = edge_maps{best_path(i + 1)};
    lines_next = [next_edge_map(1: end - 1, :),  next_edge_map(2: end, :)];
    export = vertical_to_line(this_farm.export, lines);
    entrance = vertical_to_line(next_farm.entrance, lines_next);
    debug_line = extend_line([export([3 4]), entrance([3 4])], 4);
    
    
    intersections_of_edges = sortrows([debug_line([1 2]);...
        cell2mat(arrayfun(@(i) get_intersections(debug_line, edges_to_lines(i{1})), edge_maps, 'UniformOutput', false));...
        debug_line([3 4])]);
    
    intersections_count = length(intersections_of_edges);
    inter_lines = [intersections_of_edges(1:2:intersections_count, :), intersections_of_edges(2:2:intersections_count, :)];
    
    arrayfun(@(i) plot(inter_lines(i, [1 3]), inter_lines(i, [2 4]), 'green', 'LineWidth', 2), 1:length(inter_lines(:, 1)));
    
    for j = 1:length(edge_maps)
        debug_edge_map = edge_maps{j};
        debug_lines = [debug_edge_map(1: end - 1, :),  debug_edge_map(2: end, :)];
        points = get_intersections(debug_line, debug_lines);
        if length(points) >= 2
            lines = cross_path(points(1, :), points(2, :), debug_lines);
            plot_lines = [lines(:, [1 2])', lines(end, [3 4])'];
            plot(plot_lines(1, :), plot_lines(2, :), 'green', 'LineWidth', 2);
        end
    end
    
    text(this_farm.export(1), this_farm.export(2), "Exit" + best_path(i) + " Farm" + i)
    text(this_farm.entrance(1), this_farm.entrance(2), "Entrance" + best_path(i) + " Farm" + i)
end


function height_map = get_height_map(path, earth_redius, origin)
import transform.longlat2meter;
[x, y, z] = importMap(path);
heigt_locations = longlat2meter([x, y]', earth_redius, origin);
x_meter = heigt_locations(1, :)';
y_meter = heigt_locations(2, :)';
height_map = [x_meter, y_meter, z];
end

function edge_map = get_edge_map(path, earth_redius, origin)
import transform.longlat2meter;
[edge_x, edge_y] = importEdge(path);
edge_points = longlat2meter([edge_x, edge_y]', earth_redius, origin);
edge_map = edge_points';
end

function solution = get_single(height_map, edge_map, step, figure_num)
import transform.longlat2meter;
import opitimization.single_opitimization;
[angle, len, entrance, export] = single_opitimization(height_map, edge_map, step, figure_num);
solution = [angle, len, entrance, export];
end

function res = cross_path(point1, point2, lines)
    function line = line_of_point(point, lines)
        for i = 1:length(lines)
            if in_line(point, lines(i, :))
                line = lines(i, :);
                break;
            end
        end
    end

    function res = forward(start_line, end_line, lines)
        function res = forward_line(line, lines)
            for i = 1:length(lines)
                if (lines(i, 1) == line(3)) && (lines(i, 2) == line(4))
                    res = lines(i, :);
                    break;
                end
            end
        end
        
        res = [];
        temp = start_line;
        while true
            next = forward_line(temp, lines);
            res = [res; next];
            
            if all(next == end_line)
                break
            end
            
            temp = next;
        end
    end

    function res = backward(start_line, end_line, lines)
        function res = backward_line(line, lines)
            for i = 1:length(lines)
                if (lines(i, 3) == line(1)) && (lines(i, 4) == line(2))
                    res = lines(i, :);
                    break;
                end
            end
        end
        
        res = [start_line;];
        temp = start_line;
        while true
            next = backward_line(temp, lines);
            if all(next == end_line)
                break;
            end
            res = [res; next];
            temp = next;
        end
    end

start_line = line_of_point(point1, lines);
end_line = line_of_point(point2, lines);

backward_lines = [[point1, start_line([1 2])];backward(start_line, end_line, lines); [end_line([3 4]), point2]];
forword_lines = [[point1, start_line([3 4])];forward(start_line, end_line, lines); [end_line([1 2]), point2]];

if sum(vecnorm(backward_lines(:, [1 2]))) > sum(vecnorm(forword_lines(:, [1 2])))
    res = forword_lines;
else
    res = backward_lines;
end
end

function points = get_intersections(line, lines)
    function point = get_intersection(line1, line2)
        x0 = line1(1);
        x1 = line1(3);
        
        y0 = line1(2);
        y1 = line1(4);
        
        x2 = line2(1);
        x3 = line2(3);
        
        y2 = line2(2);
        y3 = line2(4);
        
        y = ((y0-y1)*(y3-y2)*x0 + (y3-y2)*(x1-x0)*y0 + (y1-y0)*(y3-y2)*x2 + (x2-x3)*(y1-y0)*y2 ) / ( (x1-x0)*(y3-y2) + (y0-y1)*(x3-x2));
        x = x2 + (x3-x2)*(y-y2) / (y3-y2);
        
        if in_line_area([x, y], line1) && in_line_area([x, y], line2)
            point = [x, y];
        else
            point = [];
        end
    end
all_points = select(arrayfun(@(i) get_intersection(line, lines(i, :)),...
    1:length(lines), "UniformOutput", false), @(i) length(i) > 1);
if length(all_points) >= 2
    points_no_order = sortrows(cell2mat(all_points'), 1);
    points = points_no_order([1 end], :);
else
    points = [];
end
end

function line = vertical_to_line(point, lines)
% ҳڲ㵽Ե
    function foot = get_foot(point, line)
        k = (line(2) - line(4)) / (line(1) - line(3));
        
        
        a = -k;
        b = 1;
        c = -line(2) + k*line(1);
        m = point(1);
        n = point(2);
        foot = [(b*b*m-a*b*n-a*c)/(a*a+b*b),(a*a*n-a*b*m-b*c)/(a*a+b*b)];
        if ~in_line_area(foot, line)
            foot = [];
        end
    end

foots = arrayfun(@(i) get_foot(point, lines(i, :)), 1: length(lines), 'UniformOutput', false);
vertical_lines = arrayfun(@(i) [point, i{1}], foots, 'UniformOutput', false);
valid_lines = select(vertical_lines, @(i) length(i) > 3);
min_len = 10000;
line = vertical_lines(1);
for i = valid_lines
    len = norm([i{1}(1), i{1}(2)] - [i{1}(3), i{1}(4)]);
    line_i = i{1};
    if (len < min_len)
        min_len = len;
        line = line_i;
    end
end
end

function res = in_line_area(point, line)
res = xor(point(1) <= line(1), point(1) <= line(3)) && xor(point(2) <= line(2), point(2) <= line(4));
end

function res = in_line(point, line)
res = (((line(1) - point(1)) / (line(3) - point(1))) - ((line(2) - point(2)) / (line(4) - line(2))) <= 1e-5...
    && in_line_area(point, line));
end

function lines = edges_to_lines(edges)
lines = [edges(1: end - 1, :), edges(2: end, :)];
end

function res = extend_line(line, len)

delta_x = line(3) - line(1);
delta_y = line(4) - line(2);
x_extend = len * delta_x / (abs(delta_x) ^ 0.5 + abs(delta_y) ^ 0.5) ^ 2;
y_extend = len * delta_y / (abs(delta_y) ^ 0.5 + abs(delta_y) ^ 0.5) ^ 2;

res = [line(1) - x_extend, line(2) - y_extend, line(3) + x_extend, line(4) + y_extend];
end
